@@ -1,4 +1,28 @@
-#!/usr/bin/python
+#!/usr/bin/python3
+# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
+#
+# Copyright (C) 2010-2012 Bryce Harrington <bryce@canonical.com>
+#
+# Permission is hereby granted, free of charge, to any person obtaining
+# a copy of this software and associated documentation files (the
+# "Software"), to deal in the Software without restriction, including
+# without limitation the rights to use, copy, modify, merge, publish,
+# distribute, sublicense, and/or sell copies of the Software, and to
+# permit persons to whom the Software is furnished to do so, subject to
+# the following conditions:
+#
+# The above copyright notice and this permission notice shall be included
+# in all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+
+from __future__ import absolute_import, print_function, unicode_literals
 
 import re
 import sys
@@ -20,6 +44,31 @@ PARAMETER_FIFTH=42
 import os
 import shutil
 
+def rotated_backup(path, count=4):
+    """
+    Rename a file keeping up to <count> prior versions
+    """
+    # Rotate the old backups
+    for i in range(count, 1, -1):
+        old_path = "%s.bak.%d" %(path, i-1)
+        new_path = "%s.bak.%d" %(path, i)
+        if os.path.exists(old_path):
+            shutil.move(old_path, new_path)
+
+    new_path = old_path
+    old_path = "%s.bak" %(path)
+    if os.path.exists(old_path):
+        shutil.move(old_path, new_path)
+
+    new_path = old_path
+    if os.path.isfile(path):
+        shutil.copy(path, new_path)
+    elif os.path.isdir(path):
+        shutil.copytree(path, new_path)
+    else:
+        shutil.move(path, new_path)
+
+
 def safe_backup(path, keep_original=True):
     """
     Rename a file or directory safely without overwriting an existing
@@ -63,36 +112,43 @@ def config_update(filename, override_params=None, merge_params=None, delim='=',
     '''filename is the input file.  fileio is the output stream'''
     keys = []
     if override_params:
-        keys = override_params.keys()
+        keys = list(override_params.keys())
     if merge_params:
-        keys.extend(merge_params.keys())
+        keys.extend(list(merge_params.keys()))
     keys = list(set(keys))
     keys.sort()
 
+    print(filename)
     for line in fileinput.input(filename):
+        print(line)
         new_line = line
 
-        if merge_params:
+        if merge_params is not None:
+            print("Merging parameters")
             for key in merge_params:
-                p = re.compile("^\s*"+key+"\s*"+delim+"\s*\"(.*)\"")
+                print(" - %s" %(key))
+                p = re.compile("^\s*"+key+"\s*"+delim+"\s*(\"?)(.*)(\"?)")
                 m = p.match(line)
                 if m:
                     value = merge_params[key].replace('"', '')
                     if len(value)>0:
-                        new_line = "%s%s\"%s %s\"\n" %(key, delim, m.group(1), value)
+                        new_line = "%s%s%s%s %s%s\n" %(key, delim, m.group(1), m.group(2), value, m.group(3))
                     keys.remove(key)
 
-        if override_params:
-            for key in override_params.keys():
+        if override_params is not None:
+            print("Overriding parameters")
+            for key in list(override_params.keys()):
                 p = re.compile("^\s*"+key+"\s*"+delim)
                 if p.match(line):
                     new_line = "%s%s%s\n" %(key, delim, override_params[key])
-                    keys.remove(key)
+                    if key in keys:
+                        keys.remove(key)
 
         fileio.write(new_line)
 
     # Handle case of parameters that weren't already present in the file
     for key in keys:
+        print("Adding key %s" %(key))
         if override_params and key in override_params:
             fileio.write("%s%s%s\n" %(key, delim, override_params[key]))
         elif merge_params and key in merge_params:
@@ -112,9 +168,9 @@ if __name__ == '__main__':
         'BAR':                        'f(1&&2*$i^2) # \o/',
         'BOTH':                        '"incorrect"',
         }
-    
+
     config_update(filename, override_params, None)
     config_update(filename, None,            merge_params)
     config_update(filename, override_params, merge_params)
-    
+
     # TODO: Test for if drm.debug=0x4 and we want to set it to 0xe
